﻿using System;
using System.Drawing;
using System.Xml.Serialization;
using System.IO;

namespace Eka.Inheritance.MDIControls
{
    [SerializableAttribute()]
    public class BasicTheme
    {
        public BasicTheme()
        { 
        }

        public NavigationPanelInfo NavigationPanelInfo
        {
            get;
            set;
        }
        public SearchBoxInfo SearchBoxInfo
        {
            get;
            set;
        }

        public CollapsiblePanelInfo CollapsiblePanelInfo
        {
            get;
            set;
        }
        public TreeViewRootNodeInfo TreeViewRootNodeInfo
        {
            get;
            set;
        }
        public TreeViewNodeInfo TreeViewNodeInfo
        {
            get;
            set;
        }

        public DeviderLine DeviderLineInfo
        {
            get;
            set;
        }

        public void Save(string FileName)
        {
            if (File.Exists(FileName))
                File.Delete(FileName);

            using (var writer = new System.IO.StreamWriter(FileName))
            {
                var serializer = new XmlSerializer(this.GetType());
                serializer.Serialize(writer, this);
                writer.Flush();
            }
        }
        public static BasicTheme Load(string FileName)
        {
            if (!File.Exists(FileName))
                return null;

            using (var stream = System.IO.File.OpenRead(FileName))
            {
                var serializer = new XmlSerializer(typeof(BasicTheme));
                return serializer.Deserialize(stream) as BasicTheme;
            }
        }

        public static BasicTheme GetBasicControlsTheme()
        {
            string objectData = global::Eka.Inheritance.MDIControls.Properties.Resources.BasicTheme;
            var serializer = new XmlSerializer(typeof(BasicTheme));
            BasicTheme result;

            using (TextReader reader = new StringReader(objectData))
            {
                result =(BasicTheme) serializer.Deserialize(reader);
            }

            return result;
        }
    }

    [SerializableAttribute()]
    public class NavigationPanelInfo
    {
        public NavigationPanelInfo()
        { }
        public string BorderColor
        {
            get;
            set;
        }
        public bool DrawBorder
        {
            get;
            set;
        }
    }

    [SerializableAttribute()]
    public class SearchBoxInfo
    {
        public SearchBoxInfo()
        { }
        public string FontName
        {
            get;
            set;
        }
        public int FontSize
        {
            get;
            set;
        }
        public string FontColor
        {
            get;
            set;
        }
        public FontStyle FontStyle
        {
            get;
            set;
        }
        public string BorderColor
        {
            get;
            set;
        }
        public bool DrawBorder
        {
            get;
            set;
        }
    }

    [SerializableAttribute()]
    public class CollapsiblePanelInfo
    {
        public CollapsiblePanelInfo()
        { }

        public string FontName
        {
            get;
            set;
        }

        public int FontSize
        {
            get;
            set;
        }

        public string FontColor
        {
            get;
            set;
        }

        public FontStyle FontStyle
        {
            get;
            set;
        }
        public string BorderColor
        {
            get;
            set;
        }
        public bool DrawBorder
        {
            get;
            set;
        }
    }

    [SerializableAttribute()]
    public class TreeViewRootNodeInfo
    {
        public TreeViewRootNodeInfo()
        { }

        public string FontName
        {
            get;
            set;
        }

        public int FontSize
        {
            get;
            set;
        }

        public string FontColor
        {
            get;
            set;
        }

        public FontStyle FontStyle
        {
            get;
            set;
        }
    }

    [SerializableAttribute()]
    public class TreeViewNodeInfo
    {
        public TreeViewNodeInfo()
        { }

        public string FontName
        {
            get;
            set;
        }

        public int FontSize
        {
            get;
            set;
        }

        public string FontColor
        {
            get;
            set;
        }

        public FontStyle FontStyle
        {
            get;
            set;
        }
    }

    [SerializableAttribute()]
    public class DeviderLine
    {
        public DeviderLine()
        { }
        
        public int Width
        {
            get;
            set;
        }
        public int Height
        {
            get;
            set;
        }
        public string BorderColor
        {
            get;
            set;
        }
        public int LocationX
        {
            get;
            set;
        }       
    }

    //[SerializableAttribute()]
    //public class CommonControlsProperties
    //{
    //    public CommonControlsProperties()
    //    { }
    //    public int LocationX
    //    {
    //        get;
    //        set;
    //    }
    //    public int ControlWidth
    //    {
    //        get;
    //        set;
    //    }
    //    public string BorderColor
    //    {
    //        get;
    //        set;
    //    }
    //}
}
