﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Eka.Inheritance.MDIControls
{
    /// <summary>
    /// Custom treeview class to create Explorer style treeview control
    /// </summary>
    public partial class CustomTreeView : TreeView
    {
        #region Member variables
        ///// <summary>
        ///// Member variable to holds the control's border color
        ///// </summary>
        //private Color mBorderColor = Color.Black;
        ///// <summary>
        ///// Member varible to holds the control's border thickness
        ///// </summary>
        //private short mBorederThickness = 1;
        ///// <summary>
        ///// Member variable to holds the control's border style
        ///// </summary>
        //private ButtonBorderStyle mButtonBorderStyle = ButtonBorderStyle.Solid;

        private const int TV_FIRST = 0x1100;
        private const int TVM_SETBKCOLOR = TV_FIRST + 29;
        private const int TVM_SETEXTENDEDSTYLE = TV_FIRST + 44;
        private const int TVS_EX_DOUBLEBUFFER = 0x0004;

        /// <summary>
        /// PInvoke method to pass message to enable treeview styles
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="pszSubAppName"></param>
        /// <param name="pszSubIdList"></param>
        /// <returns></returns>
        [DllImport("uxtheme.dll", CharSet = CharSet.Unicode)]
        public extern static int SetWindowTheme(IntPtr hWnd, string pszSubAppName, string pszSubIdList);
        private const int WM_VSCROLL = 277;

        /// <summary>
        /// Win API to set DoubleBuffered = true for the treeview to avoid flickring
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="msg"></param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam);
        #endregion

        #region constructor
        /// <summary>
        /// Constructor method
        /// </summary>
        public CustomTreeView()
        {
            InitializeComponent();
            ShowLines = false;
            //Scrollable = false;  

            // Enable default double buffering processing
            SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);
            
            // Disable default CommCtrl painting on non-Vista systems
            if (!IsWinVista)
                SetStyle(ControlStyles.UserPaint, true);                
        }              
        #endregion
        
        #region Overridden methodr       

        /// <summary>
        /// override the handle created method to send message
        /// for enabling explorer style treeview
        /// </summary>
        /// <param name="e"></param>
        protected override void OnHandleCreated(EventArgs e)
        {
            base.OnHandleCreated(e);

            if (!DesignMode && Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6)
                SetWindowTheme(Handle, "explorer", null);

            UpdateExtendedStyles();
            if (!IsWinXP)
                SendMessage(Handle, TVM_SETBKCOLOR, IntPtr.Zero, (IntPtr)ColorTranslator.ToWin32(BackColor));
        }

        //protected override void WndProc(ref Message m)
        //{
        //    if (Scrollable)
        //        Scrollable = false;
        //    base.WndProc(ref m);
        //}

        /// <summary>
        /// Overridden method to hide the Horizontal scorolling bar
        /// </summary>
        protected override CreateParams CreateParams
        {
            get
            {
                CreateParams lCrateParams = base.CreateParams;
                lCrateParams.Style |= (0x8000); //   0x00200000;// 

                return lCrateParams;
            }
        }

        #region Private Method
        private void UpdateExtendedStyles()
        {
            int Style = 0;

            if (DoubleBuffered)
                Style |= TVS_EX_DOUBLEBUFFER;

            if (Style != 0)
                SendMessage(Handle, TVM_SETEXTENDEDSTYLE, (IntPtr)TVS_EX_DOUBLEBUFFER, (IntPtr)Style);
        }

        private static bool IsWinVista
        {
            get
            {
                OperatingSystem OS = Environment.OSVersion;
                return (OS.Platform == PlatformID.Win32NT) && (OS.Version.Major >= 6);
            }
        }

        private static bool IsWinXP
        {
            get
            {
                OperatingSystem OS = Environment.OSVersion;
                return (OS.Platform == PlatformID.Win32NT) &&
                    ((OS.Version.Major > 5) || ((OS.Version.Major == 5) && (OS.Version.Minor == 1)));
            }
        }
        #endregion

        #endregion    
    }
}
