﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace Eka.Inheritance.MDIControls {
    [Serializable]
    [TypeConverter(typeof(ResizeOptionConverter))]
    [ComVisible(true)]
    public struct ResizeOptions {

        #region Member Variables
        private bool mAllowResizeFromTop;
        private bool mAllowResizeFromBottom;
        private bool mAllowResizeFromLeft;
        private bool mAllowResizeFromRight;
        private bool mAllowResizeFromTopEdge;
        private bool mAllowResizeFromBottomEdge;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor method to create the object with.
        /// </summary>
        /// <param name="AllowFromLeft">Allow to resize from left edge</param>
        /// <param name="AllowFromRight">Allow to resize from right edge</param>
        /// <param name="AllowFromTop">Allow to resize from top edge</param>
        /// <param name="AllowFromBottom">Allow to resize from bottom edge</param>
        /// <param name="AllowFromTopEdge">Allow to resize from top corner edge</param>
        /// <param name="AllowFromBottomEdge">Allow to resize from bottom corner edge</param>
        public ResizeOptions(bool AllowFromLeft, bool AllowFromRight, bool AllowFromTop, bool AllowFromBottom, bool AllowFromTopEdge, bool AllowFromBottomEdge) {
            mAllowResizeFromLeft = AllowFromLeft;
            mAllowResizeFromRight = AllowFromRight;
            mAllowResizeFromTop = AllowFromTop;
            mAllowResizeFromBottom = AllowFromBottom;
            mAllowResizeFromTopEdge = AllowFromTopEdge;
            mAllowResizeFromBottomEdge = AllowFromBottomEdge;
        }
        #endregion

        #region Proeprties
        /// <summary>
        /// Get/Set properties to allowing resize from the top side.
        /// </summary>
        [DefaultValue(false)]
        public bool AllowFromTop {
            get {
                return mAllowResizeFromTop;
            }
            set {
                mAllowResizeFromTop = value;
            }
        }

        /// <summary>
        /// Get/Set properties to allowing resize from the bottom side.
        /// </summary>
        [DefaultValue(false)]
        public bool AllowFromBottom {
            get {
                return mAllowResizeFromBottom;
            }
            set {
                mAllowResizeFromBottom = value;
            }
        }

        /// <summary>
        /// Get/Set properties to allowing resize from the left side.
        /// </summary>
        [DefaultValue(false)]
        public bool AllowFromLeft {
            get {
                return mAllowResizeFromLeft;
            }
            set {
                mAllowResizeFromLeft = value;
            }
        }

        /// <summary>
        /// Get/Set properties to allowing resize from the right side.
        /// </summary>
        [DefaultValue(false)]
        public bool AllowFromRight {
            get {
                return mAllowResizeFromRight;
            }
            set {
                mAllowResizeFromRight = value;
            }
        }

        /// <summary>
        /// Get/Set properties to allowing resize from the top corner edge.
        /// </summary>
        [DefaultValue(false)]
        public bool AllowResizeFromTopEdge {
            get {
                return mAllowResizeFromTopEdge;
            }
            set {
                mAllowResizeFromTopEdge = value;
            }
        }

        /// <summary>
        /// Get/Set properties to allowing resize from the bottom corner edge.
        /// </summary>
        [DefaultValue(false)]
        public bool AllowResizeFromBottomEdge {
            get {
                return mAllowResizeFromBottomEdge;
            }
            set {
                mAllowResizeFromBottomEdge = value;
            }
        }
        #endregion End of Properties
    }
}
