﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Eka.Inheritance.MDIControls
{
    public partial class SearchBox : UserControl
    {    
        #region Member variables and constant
        // An event that clients can use to be notified whenever the
        // elements of the list change.
        public event CustomTextBox.SearchTextHandler SearchTextChanged;

        /// <summary>
        /// Constant to define the Gap between the Search text box and the Image icon
        /// </summary>
        private const short mGapBetweenTextboxAndSearchIcon = 2;
        /// <summary>
        /// Memeber variable to holds the Controls Border color (#666666)
        /// </summary>
        private Color mBorderColor = Color.LightGray; //Color.FromArgb(((int)(((byte)(102)))), ((int)(((byte)(102)))), ((int)(((byte)(102)))));
        /// <summary>
        /// Member variable to holds the Border thickness properties. by default it will be zero(0).
        /// </summary>
        private short mBorederThickness = 1;
        /// <summary>
        /// Member variable to holds the border style of the control.
        /// Default: None
        /// </summary>
        private ButtonBorderStyle mButtonBorderStyle = ButtonBorderStyle.Solid;
        /// <summary>
        /// Member variable to holds the Search button position(Left/Right)
        /// Default value: Right
        /// </summary>
        private SearchButtonPosition mSearchButtonPosition;
        /// <summary>
        /// Member variable to holds the Draw border value
        /// </summary>
        private bool mDrawBorder = true;

        private const int TV_FIRST = 0x1100;
        private const int TVM_SETBKCOLOR = TV_FIRST + 29;
        private const int TVM_SETEXTENDEDSTYLE = TV_FIRST + 44;
        private const int TVS_EX_DOUBLEBUFFER = 0x0004;

        /// <summary>
        /// Win API to set DoubleBuffered = true for the treeview to avoid flickring
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="msg"></param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam);
        #endregion

        /// <summary>
        /// Constructor method
        /// </summary>
        public SearchBox()
        {
            InitializeComponent();

            // Enable default double buffering processing
            SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);

            // Disable default CommCtrl painting on non-Vista systems
            if (!IsWinVista)
                SetStyle(ControlStyles.UserPaint, true);  

            //Initialize the basic control theme
            InitializeTheme();
        }

        /// <summary>
        /// Method to initialze the basic theme of the control
        /// TODO: Initialization of this theme from xml file
        /// </summary>
        private void InitializeTheme()
        {
            ctrCustomTextBox.Font = new System.Drawing.Font("Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            base.Font = new System.Drawing.Font("Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            //Set the button direction
            mSearchButtonPosition = SearchButtonPosition.Right;

            //Set the padding
            Padding lPadding = new Padding();
            lPadding.Left = Padding.Left < 3 ? 3 : Padding.Left;
            lPadding.Right = Padding.Right < 2 ? 2 : Padding.Right;
            lPadding.Top = Padding.Top < 2 ? 2 : Padding.Top;
            lPadding.Bottom = Padding.Bottom < 2 ? 2 : Padding.Bottom;
            Padding = lPadding;

            //Initialize the control size first
            Size = new Size(200, CalculateControlHeight());

            BackColor = Color.White;
            BorderStyle = BorderStyle.None;

            mBorederThickness = 1;
            mButtonBorderStyle = ButtonBorderStyle.Solid;

            //Set the textbox theme
            ctrCustomTextBox.Resize += OnEkaTextBoxResize;
            ctrCustomTextBox.SearchTextChanged += OnSearchItemChanged;
            ctrCustomTextBox.Dock = DockStyle.None;
            ctrCustomTextBox.BackColor = Color.White;
            ctrCustomTextBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
            ctrCustomTextBox.Location = new Point(Padding.Left, Padding.Top);
            ctrCustomTextBox.AutoCompleteMode = AutoCompleteMode.SuggestAppend;
            ctrCustomTextBox.AutoCompleteSource = AutoCompleteSource.CustomSource;

            ////Set the PIcturebox theme
            ctrPictureBox.Dock = DockStyle.None;
            ctrPictureBox.BackColor = Color.Transparent;
            ctrPictureBox.BorderStyle = BorderStyle.None;
            ctrPictureBox.SizeMode = PictureBoxSizeMode.StretchImage;
        }

        /// <summary>
        /// Method to resize the controls when textbox size changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void OnEkaTextBoxResize(object sender, EventArgs e)
        {
            ResizeControls();
        }

        void OnSearchItemChanged(SearchTextArgs eventArgs)
        {
            if (SearchTextChanged != null)
            {
                eventArgs = new SearchTextArgs();
                eventArgs.SearchText = this.Text;
                SearchTextChanged(eventArgs);
            }
        }

        #region Properties
        /// <summary>
        /// Get/Set properties for the Font
        /// </summary>
        public override Font Font
        {
            get
            {
                return base.Font;
            }

            set
            {
                base.Font = value;
                //Set the textbox font
                ctrCustomTextBox.Font = value;
                ResizeControls();
            }
        }

        /// <summary>
        /// Get/Set properties to set the search box text
        /// </summary>
        public override string Text
        {
            get
            {
                return this.ctrCustomTextBox.Text;
            }

            set
            {
                //base.Text = value;
                this.ctrCustomTextBox.Text = value;
            }
        }

        /// <summary>
        /// Get/Set properties to set the Foating text
        /// </summary>
        public string FloatingText
        {
            get
            {
                return ctrCustomTextBox.FloatingText;
            }

            set
            {
                this.ctrCustomTextBox.FloatingText = value;
            }
        }

        /// <summary>
        /// Get/Set properties for Search Button position.
        /// </summary>
        public SearchButtonPosition ButtonPosition
        {
            get 
            { 
                return mSearchButtonPosition; 
            }

            set
            {
                if (value == mSearchButtonPosition)
                    return;

                mSearchButtonPosition = value;
                ResizeControls();
            }
        }

        ///// <summary>
        ///// Get/Set properties for the Font
        ///// </summary>
        //public override Font Font
        //{
        //    get
        //    {
        //        return ctrCustomTextBox.Font;
        //    }
        //    set
        //    {                

        //        //base.Font = value;

        //        //Set the textbox font
        //        ctrCustomTextBox.Font = value;
        //        ResizeControls();
        //    }
        //}

        /// <summary>
        /// Get/Set BorderColor of the control
        /// </summary>
        public Color BorderColor
        {
            get 
            {
                return mBorderColor;
            }

            set
            {
                mBorderColor = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Get/Set the border color
        /// </summary>
        public bool DrawBorder
        {
            get 
            { 
                return mDrawBorder; 
            }
            set 
            {
                mDrawBorder = value;
                Invalidate(); 
            }
        }

        /// <summary>
        /// Get/Set Border style of the control
        /// </summary>
        public ButtonBorderStyle ButtonBorderStyle
        {
            get
            { 
                return mButtonBorderStyle;
            }

            set
            {
                mButtonBorderStyle = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Get/Set the Border thickness of the control
        /// </summary>
        public short BorederThickness
        {
            get 
            { 
                return mBorederThickness;
            }

            set
            {
                mBorederThickness = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Get/Set properties for AutoCompleteCustomSource
        /// </summary>
        public string[] AutoCompleteCustomSource
        {
            get
            {
                List<string> lStringList = new List<string>();

                foreach (string lval in ctrCustomTextBox.AutoCompleteCustomSource)
                {
                    lStringList.Add(lval);
                }

                return lStringList.ToArray();
            }

            set
            {
                ctrCustomTextBox.AutoCompleteCustomSource.AddRange(value);

            }
        }

        [Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
        public new BorderStyle BorderStyle 
        {
            get;
            set;
        }    
  
        ///// <summary>
        ///// Get/Set AutoCompleteMode properties for the control
        ///// </summary>
        //public AutoCompleteMode AutoCompleteMode
        //{
        //    get { return mEkaTextBox.AutoCompleteMode; }
        //    set { mEkaTextBox.AutoCompleteMode = value; }
        //}

        ///// <summary>
        ///// Get/Set AutoCompleteSource properties for the control
        ///// </summary>
        //public AutoCompleteSource AutoCompleteSource
        //{
        //    get { return mEkaTextBox.AutoCompleteSource; }
        //    set { mEkaTextBox.AutoCompleteSource = value; }
        //}
        #endregion

        private int CalculateControlHeight()
        {
            return ctrCustomTextBox.Height + this.Padding.Top + this.Padding.Bottom + 2;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            if (mDrawBorder)
            {
                ControlPaint.DrawBorder(e.Graphics, ClientRectangle,
                                     mBorderColor, mBorederThickness, mButtonBorderStyle,
                                     mBorderColor, mBorederThickness, mButtonBorderStyle,
                                     mBorderColor, mBorederThickness, mButtonBorderStyle,
                                     mBorderColor, mBorederThickness, mButtonBorderStyle);
            }

        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            ResizeControls();
            Invalidate();
        }

        /// <summary>
        /// Method to calculate size and position to place the child controls in proper place'
        /// and set the size
        /// </summary>
        private void ResizeControls()
        {
            Height = CalculateControlHeight();

            ctrPictureBox.Size = new Size(ctrCustomTextBox.Height, ctrCustomTextBox.Height);
            ctrCustomTextBox.Width = Width - Padding.Left - Padding.Right - ctrPictureBox.Width - mGapBetweenTextboxAndSearchIcon - 2;
            Point lTempPoint = ctrCustomTextBox.Location;

            switch (mSearchButtonPosition)
            {
                case SearchButtonPosition.Right:
                    ctrCustomTextBox.Location = new Point(Padding.Left, Padding.Top);
                    ctrPictureBox.Location = new Point(Padding.Left + ctrCustomTextBox.Width + mGapBetweenTextboxAndSearchIcon, Padding.Top);
                    break;

                case SearchButtonPosition.Left:
                    ctrPictureBox.Location = new Point(Padding.Left, Padding.Top);
                    ctrCustomTextBox.Location = new Point(Padding.Left + ctrPictureBox.Width + mGapBetweenTextboxAndSearchIcon, Padding.Top);
                    break;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnHandleCreated(EventArgs e)
        {
            base.OnHandleCreated(e);


            UpdateExtendedStyles();
            if (!IsWinXP)
                SendMessage(Handle, TVM_SETBKCOLOR, IntPtr.Zero, (IntPtr)ColorTranslator.ToWin32(BackColor));
        }

        #region Private Method New
        private void UpdateExtendedStyles()
        {
            int Style = 0;

            if (DoubleBuffered)
                Style |= TVS_EX_DOUBLEBUFFER;

            if (Style != 0)
                SendMessage(Handle, TVM_SETEXTENDEDSTYLE, (IntPtr)TVS_EX_DOUBLEBUFFER, (IntPtr)Style);
        }

        private static bool IsWinVista
        {
            get
            {
                OperatingSystem OS = Environment.OSVersion;
                return (OS.Platform == PlatformID.Win32NT) && (OS.Version.Major >= 6);
            }
        }

        private static bool IsWinXP
        {
            get
            {
                OperatingSystem OS = Environment.OSVersion;
                return (OS.Platform == PlatformID.Win32NT) &&
                    ((OS.Version.Major > 5) || ((OS.Version.Major == 5) && (OS.Version.Minor == 1)));
            }
        }
        #endregion
    }
}
