﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Drawing;

namespace Eka.Inheritance.MDIControls.FormSearchBox
{
    /// <summary>
    /// Class to use for drawing border on a control.
    /// </summary>
    public class BorderDrawer
    {
        #region Member variables
        private Color mBorderColor = Color.LightGray;
        private  int WM_NCPAINT = 0x0085;
        private  int WM_ERASEBKGND = 0x0014;
        private  int WM_PAINT = 0x000F;

        [DllImport("user32.dll")]
        static extern IntPtr GetDCEx(IntPtr hwnd, IntPtr hrgnclip, uint fdwOptions);

        [DllImport("user32.dll")]
        static extern int ReleaseDC(IntPtr hwnd, IntPtr hDC);
        #endregion

        #region Properteis
        /// <summary>
        /// Get/Set border color
        /// </summary>
        public Color BorderColor
        {
            get
            {
                return mBorderColor;
            }
            set
            {
                mBorderColor = value;
            }
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Method to draw border color for a control
        /// </summary>
        /// <param name="Msg"></param>
        /// <param name="ControlWidth"></param>
        /// <param name="ControlHeight"></param>
        public void DrawBorder(ref Message Msg, int ControlWidth, int ControlHeight)
        {
            if (Msg.Msg == WM_NCPAINT || Msg.Msg == WM_ERASEBKGND || Msg.Msg == WM_PAINT)
            {
                IntPtr lHdc = GetDCEx(Msg.HWnd, Msg.WParam, 1 | 0x0020);

                if (lHdc != IntPtr.Zero)
                {
                    Graphics lGraphics = null;
                    try
                    {
                        lGraphics = Graphics.FromHdc(lHdc);
                        Rectangle lRectangle = new Rectangle(0, 0, ControlWidth, ControlHeight);
                        ControlPaint.DrawBorder(lGraphics, lRectangle, mBorderColor, ButtonBorderStyle.Solid);
                        Msg.Result = (IntPtr)1;
                    }
                    finally
                    {
                        if (lGraphics != null)
                            lGraphics.Dispose();
                        lGraphics = null;

                        ReleaseDC(Msg.HWnd, lHdc);
                    }
                }
            }
        }
        #endregion
    }
}
